import numpy as np
import pandas as pd

import os
from whacc import utils, image_tools, analysis
from pathlib import Path
from whacc import analysis, model_maker

import numpy as np
import optuna

import lightgbm as lgb
import numpy as np
from sklearn import metrics
import pdb
from tqdm.autonotebook import tqdm
from whacc import analysis
import shutil
import h5py
import warnings
import matplotlib.pyplot as plt
from matplotlib import cm
from whacc import utils
import numpy as np
import matplotlib.pyplot as plt
import copy

from whacc.pole_tracker import PoleTracking
from whacc.touch_curation_GUI import touch_gui

################################################################################################
################################################################################################
############################ MAKE THE NEW TRAINING DATA  #######################################
################################################################################################
################################################################################################
################################################################################################

h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/Session1_FINISHED/Session1/AH1179X23052021x1_final_combined.h5'
h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/Session1/AH1179X23052021x1_final_combined.h5'
frame_nums = utils.getkey(h5, 'frame_nums')
equal_x_trial_inds = utils.equal_distance_pole_sample(h5, num_videos=3, equal_x=True)

vid_dir = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED_MP4s/Session1/'
vid_dir = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_mp4s/Session1_FINISHED_MP4s/Session1'
num_trials_per_session, start_frame, end_frame = 10, 1300, 1310
base_dir = '/Users/phil/Desktop/untitled folder'
mod_str = '/Volumes/GoogleDrive-114825029448473821206/My Drive/WhACC_DATA_FINAL_WITH_AUG/final_optuna_models/ALL_optuna_samson_all_sessions_100_trials_10_frames_each__weightedx10_V4__000_model.pkl'

utils.make_transfer_learning_data(h5, start_frame, end_frame, equal_x_trial_inds, vid_dir=vid_dir, base_dir=base_dir,
                                  final_trimmed_h5_name=None, overwrite=True, model_full_file_name=mod_str)

################################################################################################
################################################################################################
############################ CURATE THE DATA USING THE GUI TOOL,     ###########################
################################DON'T FORGET TO SAVE IT          ###############################
################################################################################################
################################################################################################
h5_ind_to_curate = 0  # go through all these curate and save
base_dir = '/Users/phil/Desktop/untitled folder'
h5_list = utils.get_files(base_dir, '*.h5')
label_read_key = 'labels'
label_write_key = 'human_labels'
touch_gui(h5_list[h5_ind_to_curate], label_read_key, image_read_key='images', label_write_key=label_write_key)

################################################################################################
################################################################################################
############################ FINAL LOADING AND APPENDING THE TL DATA ###########################
################################################################################################
################################################################################################
################################################################################################


bd = '/Users/phil/Desktop/untitled folder/'
labels_key = 'labels'
train_val_split = [7, 3]
h5_list = utils.get_files(bd, '*.h5')
tvt_x, tvt_y, tvt_fn, tvt_w = utils.load_training_and_curated_data(h5_list, labels_key, train_val_split=train_val_split,
                                                                   nan_ify_data=True, new_data_weight=2)

################################################################################################
################################################################################################
############################         TRAIN MODEL USING OPTUNA        ###########################
################################################################################################
################################################################################################
################################################################################################

import numpy as np
import pandas as pd

import os
from whacc import utils, image_tools, analysis
from pathlib import Path
from whacc import analysis

import numpy as np
import optuna

import lightgbm as lgb
import numpy as np
from sklearn import metrics
import pdb
from tqdm.autonotebook import tqdm
from whacc import analysis
import shutil
import h5py

import matplotlib.pyplot as plt
from whacc.utils import *


def reset_metrics():
    def callback(env):
        mod_dir = GLOBALS__________['mod_dir']
        edge_threshold = GLOBALS__________['edge_threshold']
        thresholds = GLOBALS__________['thresholds']
        smooth_by = GLOBALS__________['smooth_by']

        real = GLOBALS__________['tvt_y'][2]
        touch_count = np.sum(np.diff(real) == 1)
        frame_nums = GLOBALS__________['tvt_fn'][2]
        yhat = env.model.predict(GLOBALS__________['tvt_x'][2])
        yhat_proba = smooth(yhat, smooth_by)
        df = analysis.thresholded_error_types(real, yhat_proba, edge_threshold=edge_threshold,
                                              frame_num_array=frame_nums, thresholds=thresholds)
        df2 = df.iloc[:, :-2] / touch_count
        x = df2.sum(axis=1)
        err = x.min()

        GLOBALS__________['all_errors_test'].append(df)
        GLOBALS__________['metrics_test']['auc'].append(metrics.roc_auc_score(real, yhat))
        GLOBALS__________['metrics_test']['touch_count_errors_per_touch'].append(err)
        GLOBALS__________['metrics_test']['best_threshold'].append(thresholds[x.argmin()])

    return callback


def thresholded_error_types(yhat, dmat):
    ######################## SHOULDNT CHANGE
    frame_nums_val = GLOBALS__________['tvt_fn'][1]
    mod_dir = GLOBALS__________['mod_dir']
    higher_is_better = False
    edge_threshold = GLOBALS__________['edge_threshold']
    thresholds = GLOBALS__________['thresholds']
    smooth_by = GLOBALS__________['smooth_by']
    ######################## SHOULDNT CHANGE -- CALC TCerr
    yhat_proba = smooth(yhat, smooth_by)
    real = dmat.get_label() if isinstance(dmat, lgb.Dataset) else dmat
    touch_count = np.sum(np.diff(real) == 1)
    #     with HiddenPrints():
    df = analysis.thresholded_error_types(real, yhat_proba,
                                          edge_threshold=edge_threshold,
                                          frame_num_array=frame_nums_val,
                                          thresholds=thresholds)
    df2 = df.iloc[:, :-2] / touch_count
    x = df2.sum(axis=1)
    err = x.min()
    ######################## THE REST IS SAVING AND RETURN FOR LGBM EVAL
    GLOBALS__________['all_errors'].append(df)
    GLOBALS__________['metrics']['auc'].append(metrics.roc_auc_score(real, yhat))
    GLOBALS__________['metrics']['touch_count_errors_per_touch'].append(err)
    GLOBALS__________['metrics']['best_threshold'].append(thresholds[x.argmin()])
    if 'basic_info' not in list(GLOBALS__________.keys()):
        GLOBALS__________['basic_info'] = dict()
    GLOBALS__________['basic_info']['threshold'] = thresholds
    GLOBALS__________['basic_info']['higher_is_better'] = higher_is_better
    GLOBALS__________['basic_info']['edge_threshold'] = edge_threshold
    GLOBALS__________['basic_info']['touch_count'] = touch_count
    GLOBALS__________['basic_info']['real_human_predictions'] = real
    GLOBALS__________['basic_info']['smooth_by'] = smooth_by
    GLOBALS__________['basic_info']['num_optuna_trials'] = GLOBALS__________['num_optuna_trials']
    GLOBALS__________['basic_info']['early_stopping_rounds'] = GLOBALS__________['early_stopping_rounds']
    GLOBALS__________['basic_info']['num_iterations'] = GLOBALS__________['num_iterations']
    GLOBALS__________['basic_info']['num_optuna_trials'] = GLOBALS__________['num_optuna_trials']
    return "touch_count_errors", err, higher_is_better


def objective(trial):
    d = utils.load_feature_data()
    names = np.asarray(d['full_feature_names_and_neuron_nums'])
    final_feature_names_USE = list(names[d['final_selected_features_bool']])
    train_DATA = lgb.Dataset(GLOBALS__________['tvt_x'][0], label=GLOBALS__________['tvt_y'][0],
                             feature_name=final_feature_names_USE, weight=GLOBALS__________['tvt_w'][0])
    val_DATA = lgb.Dataset(GLOBALS__________['tvt_x'][1], label=GLOBALS__________['tvt_y'][1],
                           feature_name=final_feature_names_USE, reference=train_DATA,
                           weight=GLOBALS__________['tvt_w'][1])

    param = {
        "objective": "binary",
        "metric": "auc",
        "verbosity": -1,
        "boosting_type": "gbdt",
        "early_stopping_rounds": GLOBALS__________['early_stopping_rounds'],
        "num_iterations": GLOBALS__________['num_iterations'],
        "lambda_l1": trial.suggest_float("lambda_l1", 1e-8, 10.0, log=True),
        "lambda_l2": trial.suggest_float("lambda_l2", 1e-8, 10.0, log=True),
        "num_leaves": trial.suggest_int("num_leaves", 2, 256),
        "feature_fraction": trial.suggest_float("feature_fraction", 0.4, 1.0),
        "bagging_fraction": trial.suggest_float("bagging_fraction", 0.4, 1.0),
        "bagging_freq": trial.suggest_int("bagging_freq", 1, 7),
        "min_child_samples": trial.suggest_int("min_child_samples", 5, 100),
        #         'first_metric_only' : True,
    }

    ######################## INIT THE LISTS FOR SAVING LATER

    GLOBALS__________['metrics'] = dict()
    GLOBALS__________['metrics']['auc'] = []
    GLOBALS__________['metrics']['touch_count_errors_per_touch'] = []
    GLOBALS__________['metrics']['best_threshold'] = []
    GLOBALS__________['all_errors'] = []

    GLOBALS__________['metrics_test'] = dict()
    GLOBALS__________['metrics_test']['auc'] = []
    GLOBALS__________['metrics_test']['touch_count_errors_per_touch'] = []
    GLOBALS__________['metrics_test']['best_threshold'] = []
    GLOBALS__________['all_errors_test'] = []

    gbm = lgb.train(
        param,
        train_DATA,
        valid_sets=[val_DATA],
        feval=thresholded_error_types,
        callbacks=[reset_metrics()],
    )

    preds = gbm.predict(GLOBALS__________['tvt_x'][1])
    fpr, tpr, thresholds_auc = metrics.roc_curve(GLOBALS__________['tvt_y'][1], preds, pos_label=1)
    AUC_out = metrics.auc(fpr, tpr)

    # save the model
    mod_num = str(len(utils.get_files(GLOBALS__________['mod_dir'], '*_model.pkl'))).zfill(3)
    fn = GLOBALS__________['mod_dir'] + mod_num + '_model.pkl'
    utils.save_obj(gbm, fn)

    d = dict()
    d['metrics'] = GLOBALS__________['metrics']
    d['all_errors'] = GLOBALS__________['all_errors']
    d['metrics_test'] = GLOBALS__________['metrics_test']
    d['all_errors_test'] = GLOBALS__________['all_errors_test']

    fn = GLOBALS__________['mod_dir'] + mod_num + '_model_results.pkl'
    utils.save_obj(d, fn)

    if mod_num == '000':
        fn = GLOBALS__________['mod_dir'] + 'basic_info.pkl'
        utils.save_obj(GLOBALS__________['basic_info'], fn)

    return AUC_out


# bd = '/Users/phil/Desktop/untitled folder/'
# labels_key = 'labels'
# h5_list = utils.get_files(bd, '*.h5')
# tvt_x, tvt_y, tvt_fn, tvt_w = load_training_and_curated_data(h5_list, labels_key)


GLOBALS__________ = dict()
######################## model save directory
bd = '/Users/phil/Desktop/tmp_mod_test/'
study_name = 'my_custom_optuna_models'

GLOBALS__________['mod_dir'] = bd + os.sep + study_name + os.sep

GLOBALS__________['num_optuna_trials'] = 20  ########  20
GLOBALS__________['early_stopping_rounds'] = 10  ########  100
GLOBALS__________['num_iterations'] = 10000
######################## USER SETTING AFFECTING THE FINAL EVAL RESULTS
GLOBALS__________['edge_threshold'] = 5
GLOBALS__________['thresholds'] = np.linspace(0.4, .8, 5)
GLOBALS__________['smooth_by'] = 5
########################
GLOBALS__________['tvt_x'] = tvt_x
GLOBALS__________['tvt_y'] = tvt_y
GLOBALS__________['tvt_fn'] = tvt_fn

GLOBALS__________['tvt_w'] = tvt_w
########################
GLOBALS__________['study_name'] = study_name
GLOBALS__________['storage_dir'] = GLOBALS__________['mod_dir']
GLOBALS__________['basic_info'] = dict()

load_if_exists = True
mod_dir = GLOBALS__________['mod_dir']
if os.path.isdir(mod_dir):
    inp = input('directory exists already, type "delete" to delete existing training folder')
    if inp.lower() == 'delete':
        shutil.rmtree(mod_dir)
        print('directory was deleted and remade ready to run!')
    else:
        assert False, "directory already exists stopping training "

make_path(mod_dir)

if __name__ == "__main__":
    study_name = GLOBALS__________['study_name']
    storage_dir = GLOBALS__________['storage_dir']

    storage = f"sqlite:///{storage_dir}/OPTUNA_SAVE_{study_name}.db"

    study = optuna.create_study(study_name=study_name, storage=storage, direction="maximize",
                                load_if_exists=load_if_exists)

    study.optimize(objective, n_trials=GLOBALS__________['num_optuna_trials'], timeout=None, n_jobs=1,
                   show_progress_bar=True)

    print("Number of finished trials: {}".format(len(study.trials)))

    print("Best trial:")
    trial = study.best_trial

    print("  Value: {}".format(trial.value))

    print("  Params: ")
    for key, value in trial.params.items():
        print("    {}: {}".format(key, value))

################################################################################################
################################################################################################
############################         plot the best models        ###########################
################################################################################################
################################################################################################
################################################################################################
d2 = d_all
# make fig and axis
fig, ax1 = plt.subplots(figsize=[20, 5])
ax2 = ax1.twinx()
# setup colors
inds = np.linspace(.15, .95, len(loop_peaks))
Blues = cm.get_cmap('Blues')([inds])
Reds = cm.get_cmap('Reds')([inds])

color = Blues[0][i]
ax1.set_ylabel('1-auc', color=color)  # we already handled the x-label with ax1
ax1.plot(1 - d2['auc'], color=color)
ax1.tick_params(axis='y', labelcolor=color)

color = Reds[0][i]
ax2.set_xlabel('touch_count_errors_per_touch')
ax2.set_ylabel('touch_count_errors_per_touch', color=color)
ax2.plot(d2['touch_count_errors_per_touch'], color=color)
plt.text(len(d2['touch_count_errors_per_touch']), d2['touch_count_errors_per_touch'][-1], str(i))
ax2.tick_params(axis='y', labelcolor=color)


def foo_plot2(d, early_stopping_num, loop_peaks=None):
    x = np.diff(1 - d['auc'])
    peaks = np.where(x > 0.001)[0] + 1
    if loop_peaks is None:
        loop_peaks = np.diff([0] + list(peaks) + [len(d['auc'])])
    fig, ax1 = plt.subplots(figsize=[20, 5])
    ax2 = ax1.twinx()
    # tot = len(loop_peaks)

    inds = np.linspace(.15, .95, len(loop_peaks))
    Blues = cm.get_cmap('Blues')([inds])
    Reds = cm.get_cmap('Reds')([inds])

    for i, (k1, k2) in enumerate(utils.loop_segments(loop_peaks)):
        d2 = {'auc': d['auc'][k1:k2], 'touch_count_errors_per_touch': d['touch_count_errors_per_touch'][k1:k2]}
        color = Blues[0][i]
        ax1.set_ylabel('1-auc', color=color)  # we already handled the x-label with ax1
        ax1.plot(1 - d2['auc'], color=color)
        ax1.tick_params(axis='y', labelcolor=color)

        color = Reds[0][i]
        ax2.set_xlabel('touch_count_errors_per_touch')
        ax2.set_ylabel('touch_count_errors_per_touch', color=color)
        ax2.plot(d2['touch_count_errors_per_touch'], color=color)
        plt.text(len(d2['touch_count_errors_per_touch']), d2['touch_count_errors_per_touch'][-1], str(i))
        ax2.tick_params(axis='y', labelcolor=color)

        L = len(d2['auc'])

        b = np.argmin(1 - d2['auc'][-early_stopping_num:])
        b = np.arange(early_stopping_num)[b]
        a = b + L - early_stopping_num
        ax1.plot(a, 1 - d2['auc'][a], 'g.')

        b = np.argmin(d2['touch_count_errors_per_touch'][-early_stopping_num:])
        b = np.arange(early_stopping_num)[b]
        a = b + L - early_stopping_num
        ax2.plot(a, d2['touch_count_errors_per_touch'][a], 'g.')

        # b = np.argmin(np.flip(1-d2['auc'][-early_stopping_num:]))
        # b = np.flip(np.arange(early_stopping_num))[b]
        # a = b+L-early_stopping_num
        # ax1.plot(a, 1-d2['auc'][a], 'g.')
        #
        # b = np.argmin(np.flip(d2['touch_count_errors_per_touch'][-early_stopping_num:]))
        # b = np.flip(np.arange(early_stopping_num))[b]
        # a = b+L-early_stopping_num
        # ax2.plot(a, d2['touch_count_errors_per_touch'][a], 'g.')
    plt.grid()
    return ax1, ax2


def plot2(d, loop_peaks=None, early_stopping_num=None):
    from matplotlib import cm
    if early_stopping_num is None:
        early_stopping_num = 500
    x = np.diff(1 - d['auc'])
    peaks = np.where(x > 0.001)[0] + 1
    if loop_peaks is None:
        loop_peaks = np.diff([0] + list(peaks) + [len(d['auc'])])
    fig, ax1 = plt.subplots(figsize=[20, 5])
    ax2 = ax1.twinx()
    tot = len(loop_peaks)

    inds = np.linspace(.15, .95, len(loop_peaks))
    Blues = cm.get_cmap('Blues')([inds])
    Reds = cm.get_cmap('Reds')([inds])

    for i, (k1, k2) in enumerate(utils.loop_segments(loop_peaks)):
        # c1 = i+1/tot

        d2 = {'auc': d['auc'][k1:k2], 'touch_count_errors_per_touch': d['touch_count_errors_per_touch'][k1:k2]}
        color = Blues[0][i]
        ax1.set_ylabel('1-auc', color=color)  # we already handled the x-label with ax1
        ax1.plot(1 - d2['auc'], color=color)
        ax1.tick_params(axis='y', labelcolor=color)

        color = Reds[0][i]
        ax2.set_xlabel('touch_count_errors_per_touch')
        ax2.set_ylabel('touch_count_errors_per_touch', color=color)
        ax2.plot(d2['touch_count_errors_per_touch'], color=color)
        plt.text(len(d2['touch_count_errors_per_touch']), d2['touch_count_errors_per_touch'][-1], str(i))
        ax2.tick_params(axis='y', labelcolor=color)

        L = len(d2['auc'])

        b = np.argmin(np.flip(1 - d2['auc'][-early_stopping_num:]))
        b = np.flip(np.arange(early_stopping_num))[b]
        a = b + L - early_stopping_num
        ax1.plot(a, 1 - d2['auc'][a], 'g.')

        b = np.argmin(np.flip(d2['touch_count_errors_per_touch'][-early_stopping_num:]))
        b = np.flip(np.arange(early_stopping_num))[b]
        a = b + L - early_stopping_num
        ax2.plot(a, d2['touch_count_errors_per_touch'][a], 'g.')
    plt.grid()
    return ax1, ax2


def foo_best_ind(metric_in, min_or_max='min'):
    if min_or_max == 'min':
        return np.argmin([np.min(k) for k in metric_in])
    elif min_or_max == 'max':
        return np.argmax([np.max(k) for k in metric_in])
    else:
        assert False, 'oops'


def load_model_results(bd):
    results_list = []
    model_number = []
    for k in tqdm(utils.sort(utils.get_files(bd, '*_model_results.pkl'))):
        results_list.append(utils.load_obj(k))
        model_number.append(int(os.path.basename(k)[:3]))
    d_out = {}
    d_out['model_number'] = model_number
    d_out['validation'] = {}
    d_out['test'] = {}

    d_out['validation']['auc'] = np.asarray([np.asarray(k['metrics']['auc']) for k in results_list], dtype=object)
    d_out['test']['auc'] = np.asarray([np.asarray(k['metrics_test']['auc']) for k in results_list], dtype=object)
    d_out['validation']['TCE'] = np.asarray(
        [np.asarray(k['metrics']['touch_count_errors_per_touch']) for k in results_list], dtype=object)
    d_out['test']['TCE'] = np.asarray(
        [np.asarray(k['metrics_test']['touch_count_errors_per_touch']) for k in results_list], dtype=object)
    d_out['trial_lengths'] = [len(k) for k in d_out['validation']['auc']]

    d_out['validation']['best_auc_mod_ind'] = foo_best_ind(d_out['validation']['auc'], 'max')
    d_out['test']['best_auc_mod_ind'] = foo_best_ind(d_out['test']['auc'], 'max')
    d_out['validation']['best_TCE_mod_ind'] = foo_best_ind(d_out['validation']['TCE'], 'min')
    d_out['test']['best_TCE_mod_ind'] = foo_best_ind(d_out['test']['TCE'], 'min')
    d_out['best_mod_inds'] = [d_out['validation']['best_auc_mod_ind'], d_out['test']['best_auc_mod_ind'],
                              d_out['validation']['best_TCE_mod_ind'], d_out['test']['best_TCE_mod_ind']]

    d_out['best_iteration'] = []
    d_out['model_list_full_path'] = utils.sort(utils.get_files(bd, '*model.pkl'))
    for mod in tqdm(d_out['model_list_full_path']):
        mod = utils.load_obj(mod)
        d_out['best_iteration'].append(mod.best_iteration-1)
    d_out['basic_info'] = utils.load_obj(bd + '/basic_info.pkl')

    inds = np.linspace(.15, .95, len(d_out['validation']['auc']))

    d_out['colors'] = {}
    d_out['colors']['blues'] = cm.get_cmap('Blues')([inds])[0]
    d_out['colors']['reds'] = cm.get_cmap('Reds')([inds])[0]
    return d_out


bd = '/Users/phil/Desktop/tmp_mod_test/my_custom_optuna_models/'
d_all = load_model_results(bd)

utils.info(d_all)
utils.info(d_all['validation'])
utils.info(d_all['test'])
utils.info(d_all['basic_info'])

data_key = 'validation'
mod_inds_to_plot = None
plot_model_number_text = True
auto_set_y = True

def foo_plot_final(d_all, data_key='validation', mod_inds_to_plot=None, plot_model_number_text=True, auto_set_y=True):
    def best_mod_ind_location_plot(x, early_stopping_rounds):
        early_stopping_rounds+=1
        L = len(x)
        b = np.argmin(x[-early_stopping_rounds:])
        b = np.arange(early_stopping_rounds)[b]
        a = b + L - early_stopping_rounds
        return a, x[a]

    # make fig and axis
    early_stopping_rounds = d_all['basic_info']['early_stopping_rounds']
    fig, ax1 = plt.subplots(figsize=[20, 5])
    ax2 = ax1.twinx()
    d = d_all[data_key]
    if mod_inds_to_plot is None:
        mod_inds_to_plot = range(len(d['auc']))
    assert np.max(mod_inds_to_plot) <= len(
        d['auc']), """'mod_inds_to_plot' must not have inds larger than the number of models trained"""
    mod_inds_to_plot = np.sort(mod_inds_to_plot)
    color = d_all['colors']['blues']
    y_set = []
    for i in mod_inds_to_plot:
        x = 1 - d['auc'][i]
        ax1.plot(x, color=color[i])
        y_set.append(np.min(1 - d['auc'][i]))
        x1, y1 = best_mod_ind_location_plot(x, early_stopping_rounds)
        ax1.plot(x1, y1, 'g.')
        x2 = d_all['best_iteration'][i]
        if x2 == x1:
            ax1.plot(x2, y1, 'g*')
    ax1.set_ylabel('1-auc', color=color[-1])  # we already handled the x-label with ax1
    ax1.tick_params(axis='y', labelcolor=color[-1])
    if auto_set_y:
        ax1.set_ylim([np.min(y_set) * .75, np.max(y_set) * 1.05])

    color = d_all['colors']['reds']
    y_set = []
    for i in mod_inds_to_plot:
        x = d['TCE'][i]
        ax2.plot(x, color=color[i])
        y_set.append(np.min(d['TCE'][i]))
        x1, y1 = best_mod_ind_location_plot(x, early_stopping_rounds)
        ax2.plot(x1, y1, 'g.')
        x2 = d_all['best_iteration'][i]
        if x2 == x1:
            ax2.plot(x2, y1, 'g*')
        print(x1, y1)
        if plot_model_number_text:
            props = dict(boxstyle='round', facecolor='gray', alpha=0.5)

            plt.text(len(d['TCE'][i])-.9, d['TCE'][i][-1], str(d_all['model_number'][i]),  bbox=props)
    ax2.set_ylabel('touch count errors per touch', color=color[-1])
    ax2.tick_params(axis='y', labelcolor=color[-1])
    if auto_set_y:
        scale_by= 1.25
        ax2.set_ylim([np.min(y_set) * (1/scale_by), np.max(y_set) * scale_by])
    plt.xlabel('Boosting Iterations')

    return ax1, ax2


data_key = 'validation'
mod_inds_to_plot = [1, 10, 19]
plot_model_number_text = True
auto_set_y = True
ax1, ax2 = foo_plot_final(d_all, data_key=data_key, mod_inds_to_plot=mod_inds_to_plot,
               plot_model_number_text=plot_model_number_text, auto_set_y=auto_set_y)





d_all['best_iteration'][1]
np.asarray(d_all['best_iteration'])[mod_inds_to_plot]

mod = utils.load_obj(np.asarray(d_all['model_list_full_path'])[mod_inds_to_plot][0])
yhat = mod.predict(tvt_x[1])
y = tvt_y[1]


auc = metrics.roc_auc_score(y, yhat)
ax1.plot(35, 1-auc, 'r.')

# tvt_x, tvt_y, tvt_fn, tvt_w





"""
so i htink that the best model is correct which makes sense because its a pro package 
the ind is incorrect thought so I will have to plot a -1 form the best ind I THINK 
verify this...

also i need ot make the GLOBAL____ variable a class init self variable
"""









ax2.set_xlabel('touch_count_errors_per_touch')
ax2.set_ylabel('touch_count_errors_per_touch', color=color)
ax2.plot(d2['touch_count_errors_per_touch'], color=color)
plt.text(len(d2['touch_count_errors_per_touch']), d2['touch_count_errors_per_touch'][-1], str(i))
ax2.tick_params(axis='y', labelcolor=color)

select_mods_2_plot = [0, 1, 2]
d = dict()
d['auc'] = np.concatenate(np.asarray(d_all['val_auc'])[select_mods_2_plot])
d['touch_count_errors_per_touch'] = np.concatenate(np.asarray(d_all['val_TCE'])[select_mods_2_plot])
early_stopping = 20
"""

change the d['auc'] and d['touch_count_errors_per_touch']  to seperate inputs of just auc and touch_count_errors_per_touch
and then iterate over each set for val and test and done 
"""
d_all['auc'] = d_all['val_auc']

loop_peaks = d_all['trial_lengths']
early_stopping_num = basic_info['early_stopping_rounds']
# ax1, ax2 = foo_plot2(d)
ax1, ax2 = foo_plot2(d, early_stopping_num, loop_peaks=loop_peaks)

from whacc import utils

tmp1 = utils.load_obj('/Users/phil/Desktop/tmp_mod_test/my_custom_optuna_models/basic_info.pkl')
utils.info(tmp1)

################################################################################################
################################################################################################
############################         plot models actual performance        ###########################
################################## to see where it matches with the traces ########################
################################################################################################
################################################################################################
bd = '/Users/phil/Desktop/tmp_mod_test/my_custom_optuna_models/'
for mod in utils.sort(utils.get_files(bd, '*model.pkl')):
    print(mod)
    mod = utils.load_obj(mod)
    bi = mod.best_iteration
    print(bi)

mod = utils.load_obj(mod)

utils.info(mod)

mod.best_iteration
"""
fixed the matching now it aut finds the best model and I can plot the best model iteration in the final plot as well
seen above here 
"""



import os
import shutil
from natsort import natsorted, ns
from whacc import utils, image_tools, model_maker

import copy
import numpy as np
import pandas as pd
import h5py

from tqdm.autonotebook import tqdm


class feature_maker():
    def __init__(self, h5_in, frame_num_ind=None, frame_nums=None, operational_key='FD__original', disable_tqdm=False,
                 delete_if_exists=False, index_features_delete_the_rest=None):
        """

        Parameters
        ----------
        h5_in : h5 string pointing to the h4 file with the data to transform
        frame_num_ind : int referencing the frame num  ind to transform, note: if save_it is one ALL data is converted
        automatically and saved in h5. frame_num_ind only works when you call with
        frame_nums : default None, auto looks for key 'frame_nums' in h5 file or you can insert your own
        operational_key : the data array key to be transformed
        disable_tqdm : default False, when save_it is True it will show a loading bar with the progress unless set to true
        delete_if_exists : default False, when calling a function with save_it as True, you can choose to overwrite that
        data by setting this value to True

        Returns
        -------
        feature maker class

        Examples
        ________

        h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'

        FM = feature_maker(h5_in, frame_num_ind=2, delete_if_exists = True) #set frame_num_ind to an int so that we can quickly look at the transformation
        data, data_name = FM.shift(5, save_it=False) # shift it 5 to the right, fills in with nan as needed, look at just the 5th frame num set (5th video)
        # to see how it looks
        print('the below name will be used to save the data in the the h5 when calling functions with save_it=True')
        print(data_name)
        FM.shift(5, save_it=True) # now lets save it

        data, data_name_rolling_mean_100 = FM.rolling(100, 'mean', save_it=True)  #lets do the rolling mean and save it

        data, data_name = FM.operate('diff', kwargs={'periods': 1}, save_it=False) # lets perform a diff operation
        print(data_name)
        print(data)
        FM.operate('diff', kwargs={'periods': -1}, save_it=True) # now lets save it and save it

        # now lets change the operational key so we can transform some data we just transformed, specifically the rolling mean 10
        FM.set_operation_key(data_name_rolling_mean_100)

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=False) # lets check how the name will change
        print(data_name_diff_100_mean)
        print("notice the FD__ twice, this means the data has been transformed twice")
        print('also notice that how the data was transformed can be seen because it is split up by ____ (4 underscores)')

        data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=True) # save it

        a = utils.print_h5_keys(h5_in, 1, 1)
        key_name_list = ['FD__original', 'FD__FD__FD__original_rolling_mean_W_100_SFC_0_MP_100_____diff_periods_-50____', 'FD__FD__original_rolling_mean_W_100_SFC_0_MP_100____']
        with h5py.File(h5_in, 'r+') as h:
            for k in key_name_list:
                plt.plot(h[k][:8000, 0])
        """
        if utils.h5_key_exists(h5_in, 'has_data_been_randomized'):
            assert image_tools.get_h5_key_and_concatenate(h5_in,
                                                          'has_data_been_randomized').tolist() is False, """this data has been randomized, it is not fit to perform temporal operations on, search for key 'has_data_been_randomized' for more info"""
        self._frame_num_ind_save_ = None
        self.disable_tqdm = disable_tqdm
        self.h5_in = h5_in
        self.frame_num_ind = frame_num_ind
        self.delete_if_exists = delete_if_exists
        self.operational_key = operational_key
        if frame_nums is None:
            # print('extracting frame_nums from h5 file, ideally you should just put that in yourself though')
            frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
        self.all_frame_nums = frame_nums
        self.len_frame_nums = len(self.all_frame_nums)
        self.set_data_and_frame(frame_num_ind)
        index_features_delete_the_rest

    def set_data_inds(self, ind):
        tmp_inds = np.asarray(utils.loop_segments(self.all_frame_nums, returnaslist=True))
        if ind is None:
            self.data_inds = [tmp_inds[0][0], tmp_inds[-1][-1]]
        else:
            self.data_inds = tmp_inds[:, ind]

    def set_operation_key(self, key_name=None):
        if key_name is not None:
            self.operational_key = key_name
        with h5py.File(self.h5_in, 'r') as h:
            self.full_data_shape = np.asarray(h[self.operational_key].shape)
            if self.frame_num_ind is None:
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
            else:
                a = self.data_inds
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))

    def init_h5_data_key(self, data_key, num_f, delete_if_exists=False):
        key_exists = utils.h5_key_exists(self.h5_in, data_key)
        assert not (
                    key_exists and not delete_if_exists), "key exists, if you want to overwrite set 'delete_if_exists' = True"
        with h5py.File(self.h5_in, 'r+') as x:
            if key_exists and delete_if_exists:
                print('deleting key to overwrite it')
                del x[data_key]
            L = len(x[self.operational_key][:, 0])
            init_data = np.ones((num_f, L)) * np.nan
            # x.create_dataset_like(data_key, init_data)
            x.create_dataset(data_key, shape=[L, num_f])

    def rolling(self, window, operation, save_it=False, feature_subset_inds=None, shift_from_center=0, min_periods=None,
                kwargs={}):
        """

        Parameters
        ----------
        feature_subset_inds : select certain features of the 2048 index
        window : window size int
        operation : a string with the operation e.g. 'mean' or 'std' see pandas docs for rolling
        shift_from_center : default 0 but can shift as needed
        min_periods : default is equal to win length, only allows operation when we have this many data points. so deals with
        the edges
        save_it : bool, False, loop through frame nums and save to h5 file
        kwargs : dict of args that can be applied to 'operation' function

        Returns
        -------

        """
        if min_periods is None:
            min_periods = int(window)
        add_name_list = ['FD__' + self.operational_key + '_rolling_',
                         '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods) + '____']
        add_name_str = operation.join(add_name_list)
        if save_it:
            self._save_it_(self.rolling, window, operation, False, feature_subset_inds, shift_from_center, min_periods,
                           kwargs)
            return None, add_name_str

        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()

        for i1, i2 in utils.loop_segments(self.frame_nums):
            df_rolling = data_frame[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
            tmp_func = eval('df_rolling.' + operation)
            data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
        return data_frame, add_name_str

    def shift(self, shift_by, save_it=False, feature_subset_inds=None):
        """

        Parameters
        ----------
        feature_subset_inds : select certain features of the 2048 index
        shift_by : amount to shift by
        save_it :  bool, False, loop through frame nums and save to h5 file

        Returns
        -------

        """
        add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by) + '____'
        if save_it:
            self._save_it_(self.shift, shift_by, False, feature_subset_inds)
            return None, add_name_str

        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = data_frame[i1:i2].shift(shift_by).astype(np.float32)
        return data_frame, add_name_str

    def operate(self, operation, save_it=False, feature_subset_inds=None, extra_name_str='', kwargs={}):
        """

        Parameters
        ----------
        feature_subset_inds : select certain features of the 2048 index
        operation : a string with the operation e.g. 'mean' or 'std' or 'diff' see pandas operation on  dataframes
        save_it :  bool, False, loop through frame nums and save to h5 file
        extra_name_str : add to key name string if desired
        kwargs : dict of args that can be applied to 'operation' function see pandas operation on  dataframes

        Returns
        -------

        """
        add_name_str_tmp = self.dict_to_string_name(kwargs)
        add_name_str = 'FD__' + self.operational_key + '_' + operation + '_' + add_name_str_tmp + extra_name_str + '____'
        if save_it:
            self._save_it_(self.operate, operation, False, feature_subset_inds, extra_name_str, kwargs)
            return None, add_name_str
        if feature_subset_inds is not None:
            data_frame = self.data.copy().iloc[:, feature_subset_inds]
        else:
            data_frame = self.data.copy()

        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = eval('data_frame[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
        return data_frame, add_name_str

    @staticmethod
    def dict_to_string_name(in_dict):
        """
        used to transform dict into a string name for naming h5 keys
        Parameters
        ----------
        in_dict : dict

        Returns
        -------
        string
        """
        str_list = []
        for k in in_dict:
            str_list.append(k)
            str_list.append(str(in_dict[k]))
        return '_'.join(str_list)

    def set_data_and_frame(self, ind):
        self.frame_num_ind = ind
        if ind is None:
            self.frame_nums = copy.deepcopy(self.all_frame_nums)
        else:
            self.frame_nums = [self.all_frame_nums[ind]]
        self.set_data_inds(ind)
        self.set_operation_key()

    # tqdm(sorted(random_frame_inds[i]), disable=disable_TQDM, total=total_frames, initial=cnt1)
    def _save_it_(self, temp_funct, *args):
        self._frame_num_ind_save_ = copy.deepcopy(self.frame_num_ind)
        with h5py.File(self.h5_in, 'r+') as h:
            for k in tqdm(range(self.len_frame_nums), disable=self.disable_tqdm):
                self.set_data_and_frame(k)
                data, key_name = temp_funct(*args)
                if k == 0:
                    self.init_h5_data_key(key_name, data.shape[1], delete_if_exists=self.delete_if_exists)
                    if not self.disable_tqdm:
                        print('making key, ' + key_name)
                a = self.data_inds
                h[key_name][a[0]:a[1]] = data
        self.set_data_and_frame(self._frame_num_ind_save_)  # set data back to what it was when user set it


# class feature_maker():
#     def __init__(self, h5_in, frame_num_ind=None, frame_nums=None, operational_key='FD__original', disable_tqdm=False,
#                  delete_if_exists=False, index_features_delete_the_rest=None):
#         """
#
#         Parameters
#         ----------
#         h5_in : h5 string pointing to the h4 file with the data to transform
#         frame_num_ind : int referencing the frame num  ind to transform, note: if save_it is one ALL data is converted
#         automatically and saved in h5. frame_num_ind only works when you call with
#         frame_nums : default None, auto looks for key 'frame_nums' in h5 file or you can insert your own
#         operational_key : the data array key to be transformed
#         disable_tqdm : default False, when save_it is True it will show a loading bar with the progress unless set to true
#         delete_if_exists : default False, when calling a function with save_it as True, you can choose to overwrite that
#         data by setting this value to True
#
#         Returns
#         -------
#         feature maker class
#
#         Examples
#         ________
#
#         h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
#
#         FM = feature_maker(h5_in, frame_num_ind=2, delete_if_exists = True) #set frame_num_ind to an int so that we can quickly look at the transformation
#         data, data_name = FM.shift(5, save_it=False) # shift it 5 to the right, fills in with nan as needed, look at just the 5th frame num set (5th video)
#         # to see how it looks
#         print('the below name will be used to save the data in the the h5 when calling functions with save_it=True')
#         print(data_name)
#         FM.shift(5, save_it=True) # now lets save it
#
#         data, data_name_rolling_mean_100 = FM.rolling(100, 'mean', save_it=True)  #lets do the rolling mean and save it
#
#         data, data_name = FM.operate('diff', kwargs={'periods': 1}, save_it=False) # lets perform a diff operation
#         print(data_name)
#         print(data)
#         FM.operate('diff', kwargs={'periods': -1}, save_it=True) # now lets save it and save it
#
#         # now lets change the operational key so we can transform some data we just transformed, specifically the rolling mean 10
#         FM.set_operation_key(data_name_rolling_mean_100)
#
#         data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=False) # lets check how the name will change
#         print(data_name_diff_100_mean)
#         print("notice the FD__ twice, this means the data has been transformed twice")
#         print('also notice that how the data was transformed can be seen because it is split up by ____ (4 underscores)')
#
#         data, data_name_diff_100_mean = FM.operate('diff', kwargs={'periods': -50}, save_it=True) # save it
#
#         a = utils.print_h5_keys(h5_in, 1, 1)
#         key_name_list = ['FD__original', 'FD__FD__FD__original_rolling_mean_W_100_SFC_0_MP_100_____diff_periods_-50____', 'FD__FD__original_rolling_mean_W_100_SFC_0_MP_100____']
#         with h5py.File(h5_in, 'r+') as h:
#             for k in key_name_list:
#                 plt.plot(h[k][:8000, 0])
#         """
#         if utils.h5_key_exists(h5_in, 'has_data_been_randomized'):
#             assert image_tools.get_h5_key_and_concatenate(h5_in,
#                                                           'has_data_been_randomized').tolist() is False, """this data has been randomized, it is not fit to perform temporal operations on, search for key 'has_data_been_randomized' for more info"""
#
#         self._frame_num_ind_save_ = None
#         self.disable_tqdm = disable_tqdm
#         self.h5_in = h5_in
#         self.frame_num_ind = frame_num_ind
#         self.delete_if_exists = delete_if_exists
#         self.operational_key = operational_key
#         if frame_nums is None:
#             print('extracting frame_nums from h5 file, ideally you should just put that in yourself though')
#             frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
#         self.all_frame_nums = frame_nums
#         self.len_frame_nums = len(self.all_frame_nums)
#         self.set_data_and_frame(frame_num_ind)
#         index_features_delete_the_rest
#
#     def set_data_inds(self, ind):
#         tmp_inds = np.asarray(utils.loop_segments(self.all_frame_nums, returnaslist=True))
#         if ind is None:
#             self.data_inds = [tmp_inds[0][0], tmp_inds[-1][-1]]
#         else:
#             self.data_inds = tmp_inds[:, ind]
#
#     def set_operation_key(self, key_name=None):
#         if key_name is not None:
#             self.operational_key = key_name
#         with h5py.File(self.h5_in, 'r') as h:
#             self.full_data_shape = np.asarray(h[self.operational_key].shape)
#             if self.frame_num_ind is None:
#                 self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
#             else:
#                 a = self.data_inds
#                 self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))
#
#     def init_h5_data_key(self, data_key, delete_if_exists=False):
#         key_exists = utils.h5_key_exists(self.h5_in, data_key)
#         assert not (
#                     key_exists and not delete_if_exists), "key exists, if you want to overwrite set 'delete_if_exists' = True"
#         with h5py.File(self.h5_in, 'r+') as x:
#             if key_exists and delete_if_exists:
#                 print('deleting key to overwrite it')
#                 del x[data_key]
#             x.create_dataset_like(data_key, x[self.operational_key])
#
#     def rolling(self, window, operation, shift_from_center=0, min_periods=None, save_it=False, kwargs={}):
#         """
#
#         Parameters
#         ----------
#         window : window size int
#         operation : a string with the operation e.g. 'mean' or 'std' see pandas docs for rolling
#         shift_from_center : default 0 but can shift as needed
#         min_periods : default is equal to win length, only allows operation when we have this many data points. so deals with
#         the edges
#         save_it : bool, False, loop through frame nums and save to h5 file
#         kwargs : dict of args that can be applied to 'operation' function
#
#         Returns
#         -------
#
#         """
#         if min_periods is None:
#             min_periods = window
#         add_name_list = ['FD__' + self.operational_key + '_rolling_',
#                          '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods) + '____']
#         add_name_str = operation.join(add_name_list)
#         if save_it:
#             self._save_it_(self.rolling, window, operation, shift_from_center, min_periods, False, kwargs)
#             return None, add_name_str
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
#             tmp_func = eval('df_rolling.' + operation)
#             data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
#         return data_frame, add_name_str
#
#     def shift(self, shift_by, save_it=False):
#         """
#
#         Parameters
#         ----------
#         shift_by : amount to shift by
#         save_it :  bool, False, loop through frame nums and save to h5 file
#
#         Returns
#         -------
#
#         """
#         add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by) + '____'
#         if save_it:
#             self._save_it_(self.shift, shift_by)
#             return None, add_name_str
#
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             data_frame[i1:i2] = self.data[i1:i2].shift(shift_by).astype(np.float32)
#         return data_frame, add_name_str
#
#     def operate(self, operation, save_it=False, extra_name_str='', kwargs={}):
#         """
#
#         Parameters
#         ----------
#         operation : a string with the operation e.g. 'mean' or 'std' or 'diff' see pandas operation on  dataframes
#         save_it :  bool, False, loop through frame nums and save to h5 file
#         extra_name_str : add to key name string if desired
#         kwargs : dict of args that can be applied to 'operation' function see pandas operation on  dataframes
#
#         Returns
#         -------
#
#         """
#         add_name_str_tmp = self.dict_to_string_name(kwargs)
#         add_name_str = 'FD__' + self.operational_key + '_' + operation + '_' + add_name_str_tmp + extra_name_str + '____'
#         if save_it:
#             self._save_it_(self.operate, operation, False, extra_name_str, kwargs)
#             return None, add_name_str
#         data_frame = self.data.copy()
#         for i1, i2 in utils.loop_segments(self.frame_nums):
#             data_frame[i1:i2] = eval('self.data[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
#         return data_frame, add_name_str
#
#     @staticmethod
#     def dict_to_string_name(in_dict):
#         """
#         used to transform dict into a string name for naming h5 keys
#         Parameters
#         ----------
#         in_dict : dict
#
#         Returns
#         -------
#         string
#         """
#         str_list = []
#         for k in in_dict:
#             str_list.append(k)
#             str_list.append(str(in_dict[k]))
#         return '_'.join(str_list)
#
#     def set_data_and_frame(self, ind):
#         self.frame_num_ind = ind
#         if ind is None:
#             self.frame_nums = copy.deepcopy(self.all_frame_nums)
#         else:
#             self.frame_nums = [self.all_frame_nums[ind]]
#         self.set_data_inds(ind)
#         self.set_operation_key()
#
#     # tqdm(sorted(random_frame_inds[i]), disable=disable_TQDM, total=total_frames, initial=cnt1)
#     def _save_it_(self, temp_funct, *args):
#         self._frame_num_ind_save_ = copy.deepcopy(self.frame_num_ind)
#         with h5py.File(self.h5_in, 'r+') as h:
#             for k in tqdm(range(self.len_frame_nums), disable=self.disable_tqdm):
#                 self.set_data_and_frame(k)
#                 data, key_name = temp_funct(*args)
#                 if k == 0:
#                     self.init_h5_data_key(key_name, delete_if_exists=self.delete_if_exists)
#                     print('making key, ' + key_name)
#                 a = self.data_inds
#                 h[key_name][a[0]:a[1]] = data
#         self.set_data_and_frame(self._frame_num_ind_save_)  # set data back to what it was when user set it


def total_rolling_operation(data_in, win, operation_function, shift_from_center=0):
    """
    NOTE: for making feature data proper key names for saving is 'FD_TOTAL_' folowed by operation e.g. 'FD_TOTAL_nanstd'
    Parameters
    ----------
    data_in : 2D matrix
    win : window size
    operation_function : function to be applies to each window e.g. np.nanmean, note DON'T include parentheses
    shift_from_center : num units shift from center

    Returns
    -------
    data_out: output data
    is_nan_inds: bool array indexing where nans were
    """
    assert win % 2 == 1, 'window must be odd'
    mid = win // 2
    pad = np.zeros([win, data_in.shape[1]]) * np.nan

    L_pad = pad[:mid - shift_from_center]
    R_pad = pad[:mid + shift_from_center]

    data_in = np.vstack([L_pad, data_in, R_pad])

    is_nan_inds = []
    data_out = []
    for k in range(data_in.shape[0] - win + 1):
        x = data_in[k:(k + win)]
        data_out.append(operation_function(x))
        is_nan_inds.append(np.any(np.isnan(x)))
    return np.asarray(data_out), np.asarray(is_nan_inds)


def total_rolling_operation_h5_wrapper(FM, window, operation, key_to_operate_on, mod_key_name=None, save_it=False,
                                       shift_from_center=0):
    if save_it:
        assert mod_key_name is not None, """if save_it is True, 'mod_key_name' must not be None e.g. 'FD_TOTAL_std_1_of_'"""
    all_data = []
    with h5py.File(FM.h5_in, 'r') as h:
        frame_nums = h['frame_nums'][:]
        for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
            data_out, is_nan_inds = total_rolling_operation(h[key_to_operate_on][i1:i2, :], window, operation,
                                                            shift_from_center=shift_from_center)
            all_data.append(data_out)
    all_data = np.hstack(all_data)
    mod_key_name = mod_key_name + key_to_operate_on
    if save_it:
        utils.overwrite_h5_key(FM.h5_in, mod_key_name, all_data)
    return all_data


def convert_h5_to_feature_h5(model, in_generator, h5_new_full_file_name=None):
    assert len(in_generator.H5_file_list) == 1, 'generator must be made from a single H5 file for now can change later'
    mod_head = model_maker.load_final_model_HEAD()
    # this is due to needing to copy over the other keys like frame nums will need to combine it!!
    # see below line
    # utils.copy_over_all_non_image_keys(in_generator.H5_file_list[0], h5_new_full_file_name)
    if h5_new_full_file_name is None:
        if len(in_generator.H5_file_list) == 1:
            h5_new_full_file_name = in_generator.H5_file_list[0].replace('.h5', '_feature_data.h5')
        else:
            assert False, """if generator contains more than one file, 'h5_new_full_file_name' can not be none, please name it yourself"""

    h5c = image_tools.h5_iterative_creator(h5_new_full_file_name,
                                           overwrite_if_file_exists=True,
                                           max_img_height=1,
                                           max_img_width=2048,
                                           close_and_open_on_each_iteration=True,
                                           color_channel=False,
                                           add_to_existing_H5=False,
                                           ignore_image_range_warning=False,
                                           dtype_img=h5py.h5t.IEEE_F32LE,
                                           dtype_labels=h5py.h5t.IEEE_F32LE)
    all_pred = []
    for k in tqdm(range(in_generator.__len__())):
        x, y = in_generator.__getitem__(k)
        features = model.predict(x)
        all_pred.append(mod_head.predict(features))
        h5c.add_to_h5(features, y)
    with h5py.File(h5_new_full_file_name, 'r+') as h:
        h['FD__original'] = h['images'][:]
        del h['images']
    all_pred = np.concatenate(all_pred)
    utils.copy_over_all_non_image_keys(in_generator.H5_file_list[0], h5_new_full_file_name)
    utils.overwrite_h5_key(h5_new_full_file_name, 'CNN_pred', all_pred)


def get_feature_data_names(feature_list, n=2048):
    featuredata_names = [k.replace('FD__', '').replace('____', '') for k in feature_list]
    final_feature_names = []
    names = []
    nums = []
    for i, (k1, k2) in enumerate(zip(featuredata_names, feature_list)):
        if '_TOTAL_' in k2:
            names.append(k1)
            nums.append(np.arange(1))
            final_feature_names.append(k1)
        else:

            names.append(np.repeat(k1, n))
            nums.append(np.arange(n))
            final_feature_names.append([i1 + '_' + str(i2) for i1, i2 in zip(names[-1], nums[-1])])
    out = [np.hstack(k) for k in [final_feature_names, names, nums, featuredata_names]]
    out[0] = list(out[0])
    out[-1] = list(out[-1])
    return out


def index_out_pre_made_features(h5_in, feature_subset_inds, key):
    with h5py.File(h5_in, 'r+') as h:
        out = []
        feature_subset_inds.any()
        for k in np.where(feature_subset_inds)[0]:
            out.append(h[key][:, k])
        out = np.asarray(out).T
        del h[key]
        if feature_subset_inds.any():
            h[key] = out


def standard_feature_generation2(h5_feature_data, write_to_this_h5 = None, write_final_to_h5 = True, delete_temp_h5 = True):
    write_or_read = 'w'
    if os.path.isfile(write_to_this_h5):
        write_or_read = 'r+'
    if write_to_this_h5 is None:
        write_to_this_h5 = h5_feature_data
        write_or_read = 'r+'


    h5_new = h5_feature_data.replace('.h5', '_TEMP.h5')
    if os.path.isfile(h5_new):
        os.remove(h5_new)
    shutil.copy(h5_feature_data, h5_new)

    FM = feature_maker(h5_new, operational_key='FD__original', delete_if_exists=True, disable_tqdm=True)
    for smooth_it_by in tqdm([3, 7, 11, 15, 41]):
        data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)
    for periods in tqdm([-3, -1, 2, 3]):
        data, key_name = FM.shift(periods, save_it=True)

    all_TOTAL_keys = ['FD__FD__original_rolling_mean_W_11_SFC_0_MP_11____',
                      'FD__FD__original_rolling_mean_W_15_SFC_0_MP_15____',
                      'FD__FD__original_rolling_mean_W_3_SFC_0_MP_3____',
                      'FD__FD__original_rolling_mean_W_41_SFC_0_MP_41____',
                      'FD__FD__original_rolling_mean_W_7_SFC_0_MP_7____',
                      'FD__FD__original_shift_-1____', 'FD__FD__original_shift_-3____',
                      'FD__FD__original_shift_2____', 'FD__FD__original_shift_3____',
                      'FD__original']
    win = 1
    # key_to_operate_on = 'FD__original'
    op = np.std
    mod_key_name = 'FD_TOTAL_std_' + str(win) + '_of_'
    for key_to_operate_on in tqdm(all_TOTAL_keys):
        data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name=mod_key_name, save_it=True)

    d = utils.load_feature_data()
    bool_2048_features_index = d['final_selected_features_bool'][41:]
    feature_list_unaltered_regular = d['feature_list_unaltered'][41:]

    # exclude set that are fully created already here when you have time to code it
    cnt_ind = -1
    for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
        cnt_ind += 1
        feature_subset_inds = bool_2048_features_index[cnt_ind * 2048:(1 + cnt_ind) * 2048]
        if feature_subset_inds.any():
            data, key_name = FM.operate('diff', feature_subset_inds=feature_subset_inds, kwargs={'periods': periods},
                                        save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        cnt_ind += 1
        feature_subset_inds = bool_2048_features_index[cnt_ind * 2048:(1 + cnt_ind) * 2048]
        if smooth_it_by in [3, 7, 11, 15, 41]:
            key = feature_list_unaltered_regular[cnt_ind]
            index_out_pre_made_features(FM.h5_in, feature_subset_inds, key)
        elif feature_subset_inds.any():
            data, key_name = FM.rolling(smooth_it_by, 'mean', feature_subset_inds=feature_subset_inds, save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        cnt_ind += 1
        feature_subset_inds = bool_2048_features_index[cnt_ind * 2048:(1 + cnt_ind) * 2048]
        if feature_subset_inds.any():
            data, key_name = FM.rolling(smooth_it_by, 'std', feature_subset_inds=feature_subset_inds, save_it=True)

    for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
        cnt_ind += 1
        feature_subset_inds = bool_2048_features_index[cnt_ind * 2048:(1 + cnt_ind) * 2048]
        if periods in [-3, -1, 2, 3]:
            key = feature_list_unaltered_regular[cnt_ind]
            index_out_pre_made_features(FM.h5_in, feature_subset_inds, key)
        elif feature_subset_inds.any():
            data, key_name = FM.shift(periods, feature_subset_inds=feature_subset_inds, save_it=True)

    cnt_ind += 1
    feature_subset_inds = bool_2048_features_index[cnt_ind * 2048:(1 + cnt_ind) * 2048]
    index_out_pre_made_features(FM.h5_in, feature_subset_inds, 'FD__original')


    if write_final_to_h5:
        with h5py.File(write_to_this_h5, write_or_read) as h2:
            with h5py.File(h5_new, 'r') as h:
                L = len(h['FD__original'][:, 0])
                h2.create_dataset('final_features_2105', shape=[L, 2105])
                # final_mat = []
                x = utils.lister_it(list(h.keys()), 'FD__')
                keys = natsorted(x, alg=ns.REAL)
                cnt = 0
                for k in keys:
                    x = h[k][:]
                    if len(x.shape)==1:
                        x = x[:, None]
                    ind_add = x.shape[1]
                    h2['final_features_2105'][:, cnt:cnt+ind_add] = x
                    cnt+=ind_add
                    # final_mat.append(x)
    # final_mat = np.hstack(final_mat)
    if delete_temp_h5:
        os.remove(h5_new)
        print(h5_new+'    was deleted')

    return h5_new


def standard_feature_generation(h5_feature_data):
    FM = feature_maker(h5_feature_data, operational_key='FD__original', delete_if_exists=True)

    for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
        data, key_name = FM.shift(periods, save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)

    for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
        data, key_name = FM.operate('diff', kwargs={'periods': periods}, save_it=True)

    for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
        data, key_name = FM.rolling(smooth_it_by, 'std', save_it=True)

    win = 1
    # key_to_operate_on = 'FD__original'
    op = np.std
    mod_key_name = 'FD_TOTAL_std_' + str(win) + '_of_'
    all_keys = utils.lister_it(utils.print_h5_keys(FM.h5_in, 1, 0), 'FD__', 'FD_TOTAL')
    for key_to_operate_on in tqdm(all_keys):
        data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name=mod_key_name,
                                                      save_it=True)


def load_selected_features(h5_in, feature_index=None):
    """
    Parameters
    ----------
    h5_in : full directory of the full feature h5 file 84,009
    feature_index : bool array of len 84,009

    Returns
    -------

    """
    d = utils.load_feature_data()
    feature_list = d['feature_list_unaltered']
    if feature_index is None:
        feature_index = d['final_selected_features_bool']
        # feature_index = utils.get_selected_features(greater_than_or_equal_to=4)
    feature_index = list(feature_index)

    if isinstance(h5_in, list):
        all_x = []
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_selected_features(k, feature_list)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        all_y = np.hstack(all_y)
        return all_x

    all_x = None
    for k in tqdm(feature_list):
        if all_x is None:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
            if len(x.shape) > 1:
                inds = feature_index[:x.shape[1]]
                del feature_index[:x.shape[1]]

            else:
                inds = feature_index.pop(0)  # single true or false to include the "TOTAL" variables
            if np.any(inds):
                if len(x.shape) == 1:
                    x = x[:, None]
                all_x = copy.deepcopy(x)
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
            if len(x.shape) > 1:
                inds = feature_index[:x.shape[1]]
                del feature_index[:x.shape[1]]
                x = x[:, inds]
                all_x = np.hstack((all_x, x))
            else:
                inds = feature_index.pop(0)  # single true or false to include the "TOTAL" variables
                if inds:
                    all_x = np.hstack((all_x, x[:, None]))
    return all_x

# def load_selected_features(h5_in, feature_index=None):
#     """
#
#     Parameters
#     ----------
#     h5_in : full directory of the full feature h5 file 84,009
#     feature_index : bool array of len 84,009
#
#     Returns
#     -------
#
#     """
#     d = utils.load_feature_data()
#     feature_list = d['feature_list_unaltered']
#     if feature_index is None:
#         feature_index = d['final_selected_features_bool']
#         # feature_index = utils.get_selected_features(greater_than_or_equal_to=4)
#     feature_index = utils.make_list(feature_index)
#
#     if isinstance(h5_in, list):
#         all_x = []
#         all_y = []
#         for k in h5_in:
#             tmp_x, tmp_y = load_selected_features(k, feature_list)
#             all_x.append(tmp_x)
#             all_y.append(tmp_y)
#             del tmp_x, tmp_y
#
#         all_x = np.vstack(all_x)
#         all_y = np.hstack(all_y)
#         return all_x
#
#     all_x = None
#     for k in tqdm(feature_list):
#         if all_x is None:
#             all_x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
#             inds = feature_index[:all_x.shape[1]]
#             del feature_index[:all_x.shape[1]]
#             all_x = all_x[:, inds]
#         else:
#             x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
#             if len(x.shape) > 1:
#                 inds = feature_index[:x.shape[1]]
#                 del feature_index[:x.shape[1]]
#                 x = x[:, inds]
#                 all_x = np.hstack((all_x, x))
#             else:
#                 inds = feature_index.pop(0)  # single true or false to include the "TOTAL" variables
#                 if inds:
#                     all_x = np.hstack((all_x, x[:, None]))
#     return all_x



import pandas as pd
from natsort import natsorted, ns
import matplotlib.pyplot as plt
import h5py
import numpy as np
from whacc import utils, image_tools
import os

test_set_to = 10
from natsort import natsorted, ns


#############

#############

#############

#############
def foo1(mod_dir):
    mods = utils.get_files(mod_dir, '*')
    mods = natsorted(mods, alg=ns.REAL)[:test_set_to]
    split_impotance = []
    gain_importance = []
    for k in mods:
        lgbm = utils.load_obj(k)
        split_impotance.append(lgbm.feature_importance(importance_type='split'))
        gain_importance.append(lgbm.feature_importance(importance_type='gain'))
    return np.asarray(split_impotance), np.asarray(gain_importance)


bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/'
for folder in next(os.walk(bd))[1]:
    if '10_sets_frame' in folder:
        d = bd + folder
        count1, gain1 = foo1(d)

mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V1/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V2/'
# mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2/'
# mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num3/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num1/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num2/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num3/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num4/'


def foo1(mod_dir):
    mods = utils.get_files(mod_dir, '*')
    mods = natsorted(mods, alg=ns.REAL)[:test_set_to]
    split_impotance = []
    gain_importance = []
    for k in mods:
        lgbm = utils.load_obj(k)
        split_impotance.append(lgbm.feature_importance(importance_type='split'))
        gain_importance.append(lgbm.feature_importance(importance_type='gain'))
    return np.asarray(split_impotance), np.asarray(gain_importance)


split_impotance, gain_importance = foo1(mod_dir)

features_out_of_10 = np.sum(np.asarray(split_impotance > 0), axis=0)
total_times_used = np.sum(np.asarray(split_impotance), axis=0)
mean_gain = np.mean(gain_importance, axis=0)
max_gain = np.max(gain_importance, axis=0)
min_gain = np.min(gain_importance, axis=0)

for k in range(10):
    print(np.round(100 * np.mean(features_out_of_10 > k), 2), '% have ', k + 1, ' or more')
    print('count ', np.sum(features_out_of_10 > k))

####
next_set_of_features = np.any(gain_importance > 2, axis=0)
np.mean(next_set_of_features), np.sum(next_set_of_features), len(next_set_of_features)

###
#  remove these features
condition_1 = features_out_of_10 <= 444444  # must be used by at least 2 models out of 10 (choosing the inverse)
np.mean(condition_1)

condition_2 = mean_gain <= 2
np.mean(condition_2)

all_conditions = (condition_1 * 1 + condition_2 * 1) == 2
np.mean(all_conditions)

next_set_of_features = np.invert(all_conditions)
np.sum(next_set_of_features), len(next_set_of_features)

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/feature_index_for_feature_selection'
# set1 = np.load(bd+os.sep+'non_zero_features_bool_29913_features.npy')
# set1 = np.load(bd+os.sep+'used_more_than_once_mean_gain_mre_than_0dot1_features_bool_18445_features.npy')
# set1 = np.load(bd+os.sep+'used_more_than_NONE_times_mean_gain_mre_than_1_features_bool_6390_features.npy')
# set1 = np.load(bd+os.sep+'only_gain_more_than_2_features_bool_4353_features.npy')
# set1 = np.load(bd+os.sep+'only_gain_more_than_2_features_bool_3624_features.npy')
set1 = np.load(bd + os.sep + 'only_gain_more_than_2_features_bool_2105_features.npy')

set1_inds = np.where(set1)[0]
assert len(set1_inds) == len(all_conditions), 'wrong stuff is loaded'

next_set_of_features_inds = set1_inds[next_set_of_features]

save_feature_inds_bool = [k in next_set_of_features_inds for k in range(len(set1))]

# np.save(bd+os.sep+'used_more_than_once_mean_gain_mre_than_0dot1_features_bool_18445_features.npy', save_feature_inds_bool)
np.save(bd + os.sep + 'only_gain_more_than_2_features_bool_2073_features.npy', save_feature_inds_bool)

"""
##################################################################################################
##################################################################################################
"""

d = '/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/feature_data/feature_data_dict.pkl'
d = utils.load_obj(d)
utils.get_dict_info(d)
x = np.asarray(d['full_feature_names'])
features_used_name = x[d['final_selected_features_bool']]
x = np.unique(features_used_name)
x = natsorted(x, alg=ns.REAL)
for k in x:
    print(k)
    print(np.sum(k == features_used_name))

tmp1, inds = utils.lister_it(x, keep_strings='TOTAL', return_bool_index=True)
for k in np.where(inds)[0]:
    print(tmp1[k])
    print(mean_gain[k])

tmp2 = utils.lister_it(x, remove_string='TOTAL')

for k in tmp1:
    kk = k.replace('FD_TOTAL_std_1_of_', '')
    kk in tmp2

"""
find the max I need to make
"""
# I need to make at least 10 total feature

tmp3 = []
for k in tmp1:
    tmp3.append(k.replace('FD_TOTAL_std_1_of_', ''))

for k in tmp2:
    if k not in tmp3:
        print(k)
        print(np.sum(k == features_used_name))
    else:
        print('++++++++++++++++++++')
        print(k)
        print(np.sum(k == features_used_name))
        print('____________________')

# for k in tmp1['feature_list_short_type']:
#     print(k)
"""
##################################################################################################
##################################################################################################
"""

plt.hist(mean_gain, bins=np.linspace(0, 40, 100))
# plt.yscale('log')
"""
##################################################################################################
##################################################################################################
"""

mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/'
all_mods = []
# os.listdir(mod_dir)
folder_names = ['top_features_greter_than_3_out_of_10_3095_features',
                '10_sets_frame_nums_split_then_random_split_method_numpy_ORIGINAL_2048_just_grabbed_the_last_2048',
                '10_sets_frame_nums_split_then_random_split_method',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V1',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V2',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num3',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num4',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num5',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num1',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num2',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num3',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num4',
                '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num5']

for d in folder_names:
    x = utils.get_files(os.path.join(mod_dir, d), '*.pkl')
    x = natsorted(x, alg=ns.REAL)
    all_mods.append(np.asarray(x))

L = len(folder_names) + 2
ind = np.arange(L)  # the x locations for the groups
width = 1 / L

folder_names = ['OLD METHOD 3095 features OVERFIT',
                'Original 2,048',
                'full 84,009 features (gain plus count filter)_1',
                '29,913 features (gain plus count filter)_2',
                '18,445 features (gain plus count filter)_3',
                '6,730 features (gain plus count filter)_4.1',
                '6,390 features (only gain filter)_4.2',
                '10,126 features (only gain filter)_4.3',
                '12,481 features (gain ANY greater than filter)_4.4',
                '15,019 features (only gain filter 0.1)_4.5',
                '4,353 feature (only gain >2)',
                '3,624 feature (only gain >2)',
                '3,029 feature (only gain >2)',
                '2,105 feature (only gain >2)',
                '2,073 feature (only gain >2)', ]

fig, ax = plt.subplots()
for i, d in enumerate(all_mods):
    y = []
    for k in d:
        m = utils.load_obj(k)
        y.append(m.best_score['valid_0']['auc'])
    y.append(np.mean(y))
    ax.bar(np.arange(len(y)) + width * i - .35, 1 - np.asarray(y), width, label=folder_names[i])
    # plt.plot(1-np.asarray(y), '.', label = tmp1[i])

plt.legend(loc=1)
plt.ylabel('1 - AUC')
plt.title('feature selection models final AUC for each subset ')
# plt.ylim([0, 0.002])


x = np.arange(11).astype(int)
ax.set_xticks(x)
new_labels = ['mod ' + str(k + 1) for k in range(11)]
new_labels[-1] = 'mean models'
ax.set_xticklabels(new_labels)
"""
##################################################################################################
########################   only selected models swarmplot plot   ################################################
"""

mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/'
all_mods = []
# os.listdir(mod_dir)
folder_names = [
    # 'top_features_greter_than_3_out_of_10_3095_features',
    '10_sets_frame_nums_split_then_random_split_method_numpy_ORIGINAL_2048_just_grabbed_the_last_2048',
    '10_sets_frame_nums_split_then_random_split_method',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V1',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V2',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3',
    # '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2',
    # '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num3',
    # '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num4',
    # '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num5',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num1',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num2',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num3',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num4',
    '10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num5']

for d in folder_names:
    x = utils.get_files(os.path.join(mod_dir, d), '*.pkl')
    x = natsorted(x, alg=ns.REAL)
    all_mods.append(np.asarray(x))

L = len(folder_names) + 2
ind = np.arange(L)  # the x locations for the groups
width = 1 / L

names = [
    # 'OLD METHOD 3095 features OVERFIT',
    'Original\n2,048',
    'full\n84,009',
    '29,913',
    '18,445',
    '6,730',
    # '6,390',
    # '10,126',
    # '12,481',
    # '15,019',
    '4,353',
    '3,624',
    '3,029',
    '2,105',
    '2,073', ]

# fig, ax = plt.subplots()
import seaborn as sns

all_mod_list = []
AUC = []
num_features = np.repeat(names, 10)
mod_number = list(range(10)) * len(all_mods)
for i, d in enumerate(all_mods):
    for k in d:
        m = utils.load_obj(k)
        AUC.append(m.best_score['valid_0']['auc'])



df = {'1 - AUC': 1 - np.asarray(AUC), 'num_features': num_features, 'mod_num': mod_number}
df = pd.DataFrame(df)


ax = sns.swarmplot(y="1 - AUC",
                   x="num_features",
                   data=df,
                   color="gray", edgecolor="gray", label = 'individual models'
                   )

df_means = df.groupby("num_features", sort=False)["1 - AUC"].agg("mean").reset_index()
means = np.asarray(df_means['1 - AUC'])

plt.scatter(np.arange(len(means)), means, color = 'k', marker = '_', zorder=100, label='mean')

plt.ylim([0, 0.0023])

handles, labels = plt.gca().get_legend_handles_labels()
by_label = dict(zip(labels, handles))
plt.legend(by_label.values(), by_label.keys())
# plt.legend(bbox_to_anchor=(.8, 1.85), loc=2, borderaxespad=0, fontsize=10)



#
# sns.swarmplot(y="AUC", data=df)
#
# tips = sns.load_dataset("tips")
# sns.swarmplot(x="day", y="total_bill", data=tips)
#
# type(tips['total_bill'][:])
# type(df['AUC'][:])
# plt.plot(df['AUC'][:])
# # plt.legend(loc=1)
# # plt.ylabel('1 - AUC')
# # plt.title('feature selection models final AUC for each subset ')
# # # plt.ylim([0, 0.002])
# #
# #
# # x = np.arange(11).astype(int)
# # ax.set_xticks(x)
# # new_labels = ['mod '+str(k+1) for k in range(11)]
# # new_labels[-1] = 'mean models'
# # ax.set_xticklabels(new_labels)

"""
##################################################################################################
##################################################################################################
################## below I tried using a clustering algo to selec features #######################
####################### it didnt work very well so I ditched it ##################################
"""

to_cluster = np.vstack([features_out_of_10, total_times_used, mean_gain, max_gain, min_gain])

from sklearn import cluster
import copy
import seaborn as sns

X = copy.deepcopy(to_cluster.T)
model = cluster.KMeans(n_clusters=2, random_state=0)
model.fit(X)
utils.get_class_info(model)

tmp1 = cluster.AgglomerativeClustering().fit(X)
utils.get_class_info(tmp1)

plt.scatter(X[:, 0], X[:, 1], c=cluster.labels_, cmap='rainbow')

from sklearn.manifold import TSNE
from sklearn.preprocessing import StandardScaler

X = copy.deepcopy(to_cluster.T)
X = StandardScaler().fit_transform(X)
tsne = TSNE(n_components=2, perplexity=400, learning_rate=100, early_exaggeration=40, n_iter=250)
X_embedded = tsne.fit_transform(X)
utils.get_class_info(X_embedded)
sns.scatterplot(X_embedded[:, 0], X_embedded[:, 1])

agg_mod = cluster.AgglomerativeClustering(n_clusters=5).fit(X_embedded)
agg_mod.labels_

X = copy.deepcopy(to_cluster.T)
agg_mod = cluster.AgglomerativeClustering(n_clusters=50).fit(X)
labels = agg_mod.labels_
for k in np.unique(labels):
    if np.sum(k == labels) > 10:
        print(np.sum(k == labels))
        print(np.mean(features_out_of_10[k == labels]))
        print(np.mean(total_times_used[k == labels]))
        print(np.mean(mean_gain[k == labels]))
        print(np.mean(max_gain[k == labels]))
        print(np.mean(min_gain[k == labels]))
        print('___________')
features_out_of_10 = np.sum(np.asarray(split_impotance > 0), axis=0)
total_times_used = np.sum(np.asarray(split_impotance), axis=0)
mean_gain = np.mean(gain_importance, axis=0)
max_gain = np.max(gain_importance, axis=0)
min_gain = np.min(gain_importance, axis=0)

model = cluster.KMeans(n_clusters=5)
# fit the model
model.fit(X_embedded)
# assign a cluster to each example
yhat = model.predict(X_embedded)

sns.scatterplot(X_embedded[:, 0], X_embedded[:, 1], hue=yhat)

import sklearn

gm = sklearn.mixture.GaussianMixture(n_components=10, random_state=0).fit(X_embedded)
yhat = gm.predict(X_embedded)

import pandas as pd

fig, ax = plt.subplots()
groups = pd.DataFrame(X_embedded, columns=['x', 'y']).assign(category=y).groupby('category')
for name, points in groups:
    ax.scatter(points.x, points.y, label=name)

ax.legend()

yhat = model.predict(X)
# retrieve unique clusters
clusters = np.unique(yhat)
# create scatter plot for samples from each cluster
for cluster in clusters:
    # get row indexes for samples with this cluster
    row_ix = np.where(yhat == cluster)
    # create scatter of these samples
    plt.scatter(X[row_ix, 0], X[row_ix, 1])
# show the plot
plt.show()

# utils.get_class_info(tmp1)
# k-means clustering
from numpy import unique
from numpy import where
from sklearn.datasets import make_classification
from sklearn.cluster import KMeans
from matplotlib import pyplot

# define dataset
X, _ = make_classification(n_samples=1000, n_features=2, n_informative=2, n_redundant=0, n_clusters_per_class=1,
                           random_state=4)
# define the model
model = KMeans(n_clusters=2)
# fit the model
model.fit(X)
# assign a cluster to each example
yhat = model.predict(X)
# retrieve unique clusters
clusters = unique(yhat)
# create scatter plot for samples from each cluster
for cluster in clusters:
    # get row indexes for samples with this cluster
    row_ix = where(yhat == cluster)
    # create scatter of these samples
    pyplot.scatter(X[row_ix, 0], X[row_ix, 1])
# show the plot
pyplot.show()

# gaussian mixture clustering
from numpy import unique
from numpy import where
from sklearn.datasets import make_classification
from sklearn.mixture import GaussianMixture
from matplotlib import pyplot

# define dataset
X, _ = make_classification(n_samples=1000, n_features=2, n_informative=2, n_redundant=0, n_clusters_per_class=1,
                           random_state=4)
# define the model
model = GaussianMixture(n_components=2)
# fit the model
model.fit(X)
# assign a cluster to each example
yhat = model.predict(X)
# retrieve unique clusters
clusters = unique(yhat)
# create scatter plot for samples from each cluster
for cluster in clusters:
    # get row indexes for samples with this cluster
    row_ix = where(yhat == cluster)
    # create scatter of these samples
    pyplot.scatter(X[row_ix, 0], X[row_ix, 1])
# show the plot
pyplot.show()

################################################################################################################
################################################################################################################
################################################################################################################
################################################################################################################
################################################################################################################

from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler

x = StandardScaler().fit_transform(X)

pca = PCA(n_components=3)
principalComponents = pca.fit_transform(x)

fig = plt.figure()
ax = fig.add_subplot(projection='3d')
ax.scatter(principalComponents[:, 0], principalComponents[:, 1], principalComponents[:, 2])

finalDf = pd.concat([principalDf, pd.df[['target']]], axis=1)

fig = plt.figure(figsize=(8, 8))
ax = fig.add_subplot(1, 1, 1)
ax.set_xlabel('Principal Component 1', fontsize=15)
ax.set_ylabel('Principal Component 2', fontsize=15)
ax.set_title('2 component PCA', fontsize=20)
targets = ['Iris-setosa', 'Iris-versicolor', 'Iris-virginica']
colors = ['r', 'g', 'b']
for target, color in zip(targets, colors):
    indicesToKeep = finalDf['target'] == target
    ax.scatter(finalDf.loc[indicesToKeep, 'principal component 1']
               , finalDf.loc[indicesToKeep, 'principal component 2']
               , c=color
               , s=50)
ax.legend(targets)
ax.grid()

fn = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here_FINISHED/Jon/AH0407/160613/AH0407x160609_final_to_combine_1_to_5_of_5.h5'
fn = '/Users/phil/Desktop/pipeline_test/data_FINISHED/AH0407x160609_final_to_combine_1_to_10_of_10.h5'
fn = '/Users/phil/Desktop/feature_example2_feature_data.h5'
fn = '/Users/phil/Desktop/pipeline_test/data_FINISHED/AH0407x160609_final_to_combine_1_to_2_of_10.h5'
fn = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here_FINISHED/Jon/AH0407/160613/AH0407x160609_final_to_combine_1_to_5_of_5.h5'
from whacc import utils, image_tools
import matplotlib.pyplot as plt

utils.print_h5_keys(fn)

# d = utils.load_feature_data()
#
# all_x = []
# for k in d['feature_list_unaltered']:
#     with h5py.File(fn, 'r') as h:
#         x = h[k][:]
#         if len(x.shape) == 1:
#             x = x[:, None]
#         all_x.append(x)
# all_x = np.hstack(all_x)
# X = all_x[:, d['final_selected_features_bool']]


X = image_tools.get_h5_key_and_concatenate(fn, 'final_features_2105')
labels = image_tools.get_h5_key_and_concatenate(fn, 'labels')

mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V1/'
mod_dir = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/10_sets_frame_nums_split_then_random_split_method_numpy_feature_selection_V3_num2_v4_num4/'

mods = utils.get_files(mod_dir, '*')
# mods = natsorted(mods, alg=ns.REAL)[:test_set_to]
lgbm = utils.load_obj(mods[1])
# lgbm_shap = lgbm.predict(X, pred_contrib=True)


y_hat = lgbm.predict(X)
plt.plot(y_hat)

plt.plot(labels)

"""
clearly the files being loaded are not the same as the files being trained on
"""

arr = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]])
np.delete(arr, 1, 0), arr



from whacc import utils, image_tools
import os
import copy
import numpy as np
import matplotlib.pyplot as plt
from tqdm.auto import tqdm

from natsort import natsorted, ns

# from tensorflow.keras.models import Model
import pytz
import matplotlib.pyplot as plt
from whacc import image_tools
from whacc import utils
import os
import scipy.io as spio
import h5py
# from tqdm.notebook import tqdm

import numpy as np
import mat73
import numpy as np
import scipy.stats
from numpy import trapz

# U_file = '/Users/phil/Downloads/touch_pole.mat'
# touch_mat = utils.loadmat(U_file)
# dict_list = []
# for k in touch_mat['tStruct']:
#     dict_list.append(utils._todict(k))
# dict_list[0].keys()
#
#
# U_file = '/Users/phil/Downloads/interneurons.mat'
# # U_ARRAY = mat73.loadmat(U_file)
# touch_mat = utils.loadmat(U_file)
# dict_list = []
# for k in touch_mat['U']:
#     dict_list.append(utils._todict(k))
# d = dict_list[-1]
#
# utils.info(d)
# utils.info(d['whisker'])
# utils.info(d['meta'])
# touch_mat['U']

def mean_confidence_interval(data, confidence=0.95):
    a = 1.0 * np.array(data)
    n = len(a)
    m, se = np.mean(a), scipy.stats.sem(a)
    h = se * scipy.stats.t.ppf((1 + confidence) / 2., n-1)
    return m, m-h, m+h

def CI(data):
    out = []
    for k in data:
        out.append(mean_confidence_interval(k))
    return out

def get_h5_trial_nums(h5):
    w_file_name_nums = utils.getkey(h5, 'file_name_nums')
    w_frame_nums = utils.getkey(h5, 'frame_nums')
    trial_inds = (np.cumsum(w_frame_nums) - w_frame_nums[0]).astype(int)
    w_trials = w_file_name_nums[trial_inds]
    return w_trials

def trial_inds_to_ms_inds(frame_nums, trial_inds):
    out = []
    for i, k in enumerate(frame_nums):
        if i in trial_inds:
            out.append(np.ones(k))
        else:
            out.append(np.zeros(k))
    return np.concatenate(out).astype(bool)

def get_onset_inds(x):
    return np.where(np.diff(np.concatenate(([0], 1 * (x)))) == 1)[0]

def get_in_range_inds(pole_onset, pole_offset, frame_nums, add_onset, add_offset=0):
    inds = []
    for ii, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
        tmp1 = np.zeros(i2 - i1)
        tmp1[add_onset + pole_onset[ii]:add_offset + +pole_offset[ii]] = 1
        inds.append(tmp1)
    inds = np.concatenate(inds)
    return inds
""" steps 
1) Find my U array and determine which data is actually curated by hand for my data
2) generate the H5s (should already be done) and insert the hand contacts into the h5 files, also do this will all of samsons data
3) make a figure using foo functions for PSTH with, peak, length of time of signal, onset time, and area under curve,
set sig portion by hand
"""

U_file = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1xP-YUP9Yb9blCvX4WR5BWkeDsl3e8nKa/Phil_Iam/U_array_defined_PTITmanualCompare.mat'
U_file = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1xP-YUP9Yb9blCvX4WR5BWkeDsl3e8nKa/Phil_Iam/U_array_defined_ManualSet_18includesallcontacts.mat'
U_ARRAY = mat73.loadmat(U_file)

U_cell_nums = []
for c in U_ARRAY['U']:
    U_cell_nums.append(int(c['cellNum']))

# h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_V2_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'
# utils.open_folder(h5)
bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/'
final_h5s = utils.get_files(bd, '*final_combined.h5')

cell_list = utils.sort([1, 9, 19, 21, 77, 94])
cell_list = np.sort([int(k.split('_final_combined.h5')[0].split('x')[-1]) for k in final_h5s])
cell_list = np.intersect1d(U_cell_nums, cell_list)
# cell_list = [21]
# good_cells =[9,  19,  65,  73,  83, 103, 106, 114]
# bad_cells = [105, 94, 77, 33, 31, 23, 21, 14, 1]


h5_list = utils.sort(utils.lister_it(final_h5s, ['Session' + str(k) + '/' for k in cell_list]))
W = dict()
W['meta'] = dict()

start_pad = 100
end_pad = 350
utils.print_list_with_inds(c['varNames'])
whacc_label_key = 'yhat_by_trial_temp_model_with_nans'
whacc_label_key = 'YHAT__optuna_samson_all_sessions_8_full_trials_each__weightedx2_V1_002_model'
whacc_thresh = 0.5


def get_aligned_contacts_from_U_cell(c):
    S_ctk = np.asarray([tmp1.T.flatten() for tmp1 in c['S_ctk']])
    contacts = np.nansum([S_ctk[10], S_ctk[13]], axis=0)

def foo_operate_on_pred(x, smooth_by):
    utils.smooth(x, smooth_by)
    return x


cell_nums_tmp = [1,   9,   14,  19, 21, 23,  31, 65,  73,  77,  83,  94,  103, 105, 106, 114]
phils_onsets =  [101, 100, 110, 98, 99, 100, 90, 100, 105, 105, 109, 124, 110, 114, 111, 118]
onsets_by_hand = dict()
for i, k in enumerate(cell_nums_tmp):
    onsets_by_hand[k] =phils_onsets[i]

smooth_by = 11

for (h5, cell_num) in zip(h5_list, tqdm(cell_list)):
    if cell_num == 21:
        asdf
    cust_onset = onsets_by_hand[cell_num]

    # load the proper cell from U array and init the dict
    W[cell_num] = dict()
    cell_ind = np.where(cell_num == np.asarray(U_cell_nums))[0][0]
    c = U_ARRAY['U'][cell_ind]

    # get index for human and whacc TRIAL data (U array and H5 file respectively)
    h_trials = c['meta']['usedTrialNums'].astype(int)
    w_trials = get_h5_trial_nums(h5)
    intersecting_trials, h_trial_inds, w_trial_inds = np.intersect1d(h_trials, w_trials, return_indices=True)

    # get the index for the corresponding ms data
    h_frame_nums_temp = np.asarray([c['R_ntk'].shape[0]] * c['R_ntk'].shape[1])  # NOTE this uses data from the U array that is a 2D matrix and thus for sessions with different trials lengths you will have ot add in the frame nums here.
    h_ms_inds = trial_inds_to_ms_inds(h_frame_nums_temp, h_trial_inds)
    frame_nums = h_frame_nums_temp[h_trial_inds]  # index out proper frame_nums

    w_frame_nums_temp = utils.getkey(h5, 'frame_nums').astype(int)
    w_ms_inds = trial_inds_to_ms_inds(w_frame_nums_temp, w_trial_inds)
    w_frame_nums = w_frame_nums_temp[w_trial_inds]  # index out proper frame_nums
    assert np.all(frame_nums == w_frame_nums), "frame nums don't match this should never happen"

    # first index out all the human data we are using "h_trial_inds" for these
    pole_onset = (c['meta']['poleOnset'] * 1000).astype(int)[h_trial_inds]
    pole_offset = (c['meta']['poleOffset'] * 1000).astype(int)[h_trial_inds]

    onset_diff = int(np.median(pole_onset)- cust_onset)
    pole_onset = pole_onset-onset_diff
    pole_offset = pole_offset-onset_diff
    # pole_offset[:]= 1500

    # now do the same for WhACC data, using "w_trial_inds" for these
    "__________________"
    # pole_out_of_range_mask
    in_range = get_in_range_inds(pole_onset, pole_offset, frame_nums, add_onset=0, add_offset=0)
    # next index the human ms data using "h_ms_inds"
    # utils.overwrite_h5_key(h5, 'in_range', 'in_range')
    S_ctk = np.asarray([tmp1.T.flatten() for tmp1 in c['S_ctk']])

    h_labels = np.nansum([S_ctk[10], S_ctk[13]], axis=0)[h_ms_inds]*in_range



    spikes = c['R_ntk'].T.flatten()[h_ms_inds]*1000

    # next index the WhACC ms data using "w_ms_inds"
    ####_____####_____####_____####_____####_____####_____
    labels_tmp = utils.getkey(h5, whacc_label_key)
    resorted_inds = np.argsort(w_trials)
    segs = np.asarray(utils.loop_segments(w_frame_nums_temp, 1))
    tmp1 = []
    for i, k in enumerate(resorted_inds):
        tmp1.append(labels_tmp[segs[0, k]:segs[1, k]])
    labels_sorted = np.concatenate(tmp1)
    ####_____####_____####_____####_____####_____####_____
    w_labels = labels_sorted[w_ms_inds]*in_range
    w_labels = foo_operate_on_pred(w_labels, smooth_by)

    h_touch_onset_inds = get_onset_inds(h_labels)
    h_spk_onset = utils.cut_with_nans(spikes, h_touch_onset_inds, start_pad, end_pad=end_pad)


    w_touch_onset_inds = get_onset_inds(w_labels > whacc_thresh)
    w_spk_onset = utils.cut_with_nans(spikes, w_touch_onset_inds, start_pad, end_pad=end_pad)


    W[cell_num]['spikes'] = spikes
    W[cell_num]['w_touch_onset_inds'] = w_touch_onset_inds
    W[cell_num]['h_touch_onset_inds'] = h_touch_onset_inds
    W[cell_num]['w_spk_onset'] = w_spk_onset
    W[cell_num]['h_spk_onset'] = h_spk_onset
    W[cell_num]['h_labels'] = h_labels
    W[cell_num]['w_labels'] = w_labels

    W[cell_num]['h_spk_mean'] = np.nanmean(h_spk_onset, axis=0)
    W[cell_num]['w_spk_mean'] = np.nanmean(w_spk_onset, axis=0)
    W[cell_num]['mouseName'] = c['meta']['mouseName']
    W[cell_num]['cellName'] = c['meta']['cellName']
    W[cell_num]['sessionName'] = c['meta']['sessionName']
    W[cell_num]['usedTrialNums'] = c['meta']['usedTrialNums']
    W[cell_num]['template_img'] = utils.getkey(h5, 'template_img')


    W[cell_num]['in_range'] = in_range
    W[cell_num]['h5_name'] = h5
    W[cell_num]['frame_nums'] = w_frame_nums# frame nums are same for both, asserted above
    W[cell_num]['w_ms_inds'] = w_ms_inds



W['meta']['whacc_thresh'] = whacc_thresh
W['meta']['whacc_label_key'] = whacc_label_key
W['meta']['start_pad'] = start_pad
W['meta']['end_pad'] = end_pad
W['meta']['smooth_by'] = smooth_by
W['meta']['x'] = np.arange(end_pad+start_pad+1)-start_pad


bd= '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/small_data'
utils.save_obj(W, bd+os.sep+'W_dict_from_PSTH_V1')
# ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

def hex_to_rgb(h):
    return list(int(h[i:i+2], 16) for i in (0, 2, 4))

colors = dict()
colors['human'] = hex_to_rgb('1f77b4')
colors['whacc'] = hex_to_rgb('ff7f0e')
for k in colors:
    colors[k] = list(np.asarray(colors[k])/255)


def foo_plot(c, W, data_key, color_in, smooth_by=1, mult_by=1, leg='', ax=None):
    y = np.nanmean(c[data_key], axis=0)
    sd = np.nanstd(c[data_key], axis=0)
    se = sd/np.sqrt(c[data_key].shape[0])
    ci = 1.96*se
    plt.plot(W['meta']['x'], mult_by*utils.smooth(np.nanmean(c[data_key], axis=0), smooth_by), color=color_in+[1], label=leg, axes=ax)
    y_low, y_up = mult_by*utils.smooth(y-ci, smooth_by), mult_by*utils.smooth(y+ci, smooth_by)
    plt.fill_between(W['meta']['x'], y_low, y_up, color=color_in+[.25])
    plt.ylabel('Spk/S')
    plt.xlabel('Time (ms)')
    plt.ylim(np.max([plt.ylim()[0], 0]), plt.ylim()[1])



good_cells =[9,  19,  65,  73,  83, 103, 106, 114]
bad_cells = [105, 94, 77, 33, 31, 23, 21, 14, 1]
selected_cell = [19]

to_loop_cells = cell_list
smooth_by = 7
bd = '/Users/phil/Dropbox/HIRES_LAB/WhACC PAPER/figures/misc/PSTH_final_mod_start_to_1500/'
# utils.clear_dir(bd)
format = 'png'
transparent = True
for ii, cell_num in enumerate(to_loop_cells):

    c = copy.deepcopy(W[cell_num])
    plt.figure(figsize=[10, 6])

    c['title1'] = 'Cell '+str(cell_num)+'  smooth='+str(smooth_by)
    plt.title(c['title1'])

    foo_plot(c, W, 'h_spk_onset', colors['human'], smooth_by=smooth_by, leg='Human')
    foo_plot(c, W, 'w_spk_onset', colors['whacc'], smooth_by=smooth_by, leg='WhACC')
    plt.xlim(-50, 250)
    plt.legend()
    plt.savefig(bd+os.sep+c['title1'] + '.'+format, transparent=transparent, format=format)
    plt.close()

    plt.figure()
    plt.imshow(c['template_img'])

    plt.savefig(bd+os.sep+c['title1'] + '_POLE.'+format, transparent=transparent, format=format)
    plt.close()
utils.open_folder(bd)


# c['h_spk_on_off'] = np.arange(8, 22)
# c['w_spk_on_off'] = np.arange(7, 21)

c['h_spk_on_off_inds'] = np.intersect1d(c['h_spk_on_off'], W['meta']['x'], return_indices=True)[-1]
c['w_spk_on_off_inds'] = np.intersect1d(c['w_spk_on_off'], W['meta']['x'], return_indices=True)[-1]

c['h_sig_area'] = trapz(c['h_spk_mean'][c['h_spk_on_off_inds']])
c['w_sig_area'] = trapz(c['w_spk_mean'][c['w_spk_on_off_inds']])

c['h_sig_peak'] = np.max(c['h_spk_mean'][c['h_spk_on_off_inds']])
c['w_sig_peak'] = np.max(c['w_spk_mean'][c['w_spk_on_off_inds']])

c['h_sig_peak_ind'] = np.argmax(c['h_spk_mean'][c['h_spk_on_off_inds']])
c['w_sig_peak_ind'] = np.argmax(c['w_spk_mean'][c['w_spk_on_off_inds']])

x = c['h_spk_onset'][:, c['h_spk_on_off_inds'][c['h_sig_peak_ind']]]
c['h_sig_peak_ci_95'] = 1.96*(np.std(x)/(np.sqrt(len(x))))
x = c['w_spk_onset'][:, c['w_spk_on_off_inds'][c['w_sig_peak_ind']]]
c['w_sig_peak_ci_95'] = 1.96*(np.std(x)/(np.sqrt(len(x))))


# plt.plot(c['h_sig_peak']-c['w_sig_peak'], )


def foo_plot2(ax_, data_key, xlab, ii):
    # ax_ = ax_flat[0]
    ax_.plot(c['h'+data_key]-c['w'+data_key], ii, '.', color=colors['whacc'])
    ax_.vlines(0, *ax_.get_ylim(), color=colors['human'])
    xl = 1.5*np.max(np.abs(ax_.get_xlim()))
    ax_.set_xlim(-xl, xl)
    ax_.set_xlabel(xlab)
# foo_plot2(ax_flat[0], '_sig_peak', 'asdfasdf', ii)


fig, ax = plt.subplots(1, 6, sharey=True)
ax_flat = ax.flatten()

ax_, data_key, xlab = ax_flat[0], '_sig_peak', 'peak response\n(Spk/S)'

ax_.errorbar(0, ii, xerr=c['h_sig_peak_ci_95'])
ax_.plot(c['h'+data_key]-c['w'+data_key], ii, '.', color=colors['whacc'])
# ax_.vlines(0, *ax_.get_ylim(), color=colors['human'])
xl = 1.5*np.max(np.abs(ax_.get_xlim()))
ax_.set_xlim(-xl, xl)
ax_.set_xlabel(xlab)


'h_spk_onset'
ax_, data_key, xlab = ax_flat[1], '_spk_on_off', 'Onset Latency\n(ms)'

ax_.vlines(0, *ax_.get_ylim(), color=colors['human'])
# ax_.plot(0, *ax_.get_ylim(), '-',color=colors['human'])

ax_.plot(c['h'+data_key][0]-c['w'+data_key][0], ii, '.', color=colors['whacc'])
xl = 1.5*np.max(np.abs(ax_.get_xlim()))
ax_.set_xlim(-xl, xl)
ax_.autoscale(enable=True, axis='y', tight=True)
ax_.set_xlabel(xlab)
ax_.get_ylim()

"""
plot colored Vline at 0 for each 
we could sample 100 times at the same regions to get AUC, and peak to plot confidence intervals of 95% 
onsets cant do this unless we want to automate it
"""



c['h_sig_'] =
c['w_sig_'] =

c['h_sig_'] =
c['w_sig_'] =



# def poison_standard_error(x):
#     return 1.96*np.sqrt(np.mean(x)/len(x))
# def CI(x):
#     out = []
#     for k in x.T:
#         out.append(poison_standard_error(k))
#     return out

# plt.plot(np.nanmean(c['w_spk_onset'], axis=0), color=colors['whacc']+[1])



# plt.fill_between(W['meta']['x'], y, sd, alpha=.25)






plt.plot(np.nanmean(c['h_spk_onset'], axis=0))
plt.plot(np.nanmean(c['w_spk_onset'], axis=0))



# plt.figure()
    # plt.title(cell_num)
    # plt.plot(np.nanmean(h_spk_onset, axis=0))







bad_cells = [105, 94, 77, 33, 31, 23, 21, 14, 1]
good_cells =[9,  19,  65,  73,  83, 103, 106, 114]

for cell_num in bad_cells:
    cell_ind = np.where(cell_num == np.asarray(U_cell_nums))[0][0]
    c = U_ARRAY['U'][cell_ind]
    print(c['meta']['mouseName'], c['meta']['sessionName'])

for cell_num in good_cells:
    cell_ind = np.where(cell_num == np.asarray(U_cell_nums))[0][0]
    c = U_ARRAY['U'][cell_ind]
    print(c['meta']['mouseName'], c['meta']['sessionName'])


utils.update_whacc()



from whacc.feature_maker import feature_maker, total_rolling_operation_h5_wrapper
import matplotlib.pyplot as plt
from tqdm.auto import tqdm
import numpy as np
import h5py
import seaborn as sns
# run quoted code below to make the original mini file
import scipy
import scipy.cluster.hierarchy as sch
import pandas as pd
import os
import numpy as np
import cv2
from pathlib import Path
from PIL import Image

def cluster_corr(corr_array, max_div_by = 2):
    pairwise_distances = sch.distance.pdist(corr_array)
    linkage = sch.linkage(pairwise_distances, method='complete')
    cluster_distance_threshold = pairwise_distances.max()/max_div_by
    idx_to_cluster_array = sch.fcluster(linkage, cluster_distance_threshold, criterion='distance')
    idx = np.argsort(idx_to_cluster_array)
    if isinstance(corr_array, pd.DataFrame):
        return corr_array.iloc[idx, :].T.iloc[idx, :]
    return corr_array[idx, :][:, idx], idx

"""NOTE you will have to have both the features and the images in one H5 file"""


h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/example_feature.h5'
# h5_in = '/Users/phil/Desktop/feature_example_feature_data.h5'

# utils.print_h5_keys(h5_in)
with h5py.File(h5_in, 'r') as h:
    images = h['images'][:]
    labels = h['labels'][:]
    frame_nums = h['frame_nums'][:]
    base_name = h['base_name']
    FD__original = h['FD__original'][:]


# for div_by in [4]:#[2, 3, 4, 30, 50, 100]: I think 4 is the best for visualizing the groups of neurons
div_by = 4
FD_img = FD__original-np.min(FD__original)
FD_img = ((FD_img/np.max(FD_img))*255).astype(np.uint8)
cc = np.corrcoef(FD_img, rowvar=False)
plt.figure()
sns.heatmap(cc)
cc, sorted_inds = cluster_corr(cc, div_by) # number here sets threshold for clustering, higher results in more cluster
plt.figure()
sns.heatmap(cc)

# FD_img = FD__original-np.min(FD__original)
# FD_img = ((FD_img/np.max(FD_img))*255).astype(np.uint8)
# # tmp1 = np.concatenate(([0], np.diff(labels)==1))
# # tmp1 = labels==1
# sorted_inds = np.argsort(np.mean(FD_img[tmp1, :], axis=0))
# FD_img = FD_img[:, sorted_inds]
# cc = np.corrcoef(FD_img, rowvar=False)
# plt.figure()
# sns.heatmap(cc)

FD_img = FD_img[:, sorted_inds] # sort features by clustering

expand_by = 1000 # size of whisker and feature frames, so total will (be expand_by*2 by expand_by)
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/final_video/' + os.sep
Path(bd).mkdir(parents=True, exist_ok=True)
width = 32
height = 64

for i, (img, pole_img) in enumerate(tqdm(zip(FD_img, images), total=len(images))):
    img = img.reshape([height, width])
    img = cv2.resize(img, dsize=(expand_by, expand_by), interpolation=cv2.INTER_AREA)
    img = np.repeat(img[:, :, None], 3, axis = 2)
    pole_img = cv2.resize(pole_img, dsize=(expand_by, expand_by), interpolation=cv2.INTER_AREA)
    img = np.hstack((pole_img, img))
    im = Image.fromarray(img)
    im.save(bd+'frame'+"{:012d}".format(i)+'.png')
    if i>999999999:
        break



# bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/final_video/final_small_100_frames/'
cd_to = """cd """ + '''"'''+bd+'''"'''
mp4_file_name = 'out12.mp4'
mp4_file_name = 'final_example_vid_divby_'+str(div_by)+'.mp4'
os.system(cd_to + """ && ffmpeg -y -framerate 15 -i frame%012d.png -c:v libx264 -crf 0 """ + mp4_file_name)


#below for quick time compatible
mp4_file_name = 'quicktime_compatible_final_example_vid_divby_'+str(div_by)+'.mp4'
os.system(cd_to + """ && ffmpeg -y -framerate 20 -i frame%012d.png -vcodec libx264 -pix_fmt yuv420p -lossless 1  """ + mp4_file_name)








def rolling_mean(self, window, center = None, min_periods = None):
    window, center, min_periods, add_name_list, data_frame = self._rolling_(self, window, center = center, min_periods = min_periods)
    add_name_str = 'mean'.join(add_name_list)
    for i1, i2 in utils.loop_segments(self.frame_nums):
        df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods)
        data_frame[i1:i2] = df_rolling.mean().shift(-(center)).astype(np.float32)
    return data_frame, add_name_str



skew
sum
validate
sem             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
quantile        type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
var             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
std             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
agg             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
mean            type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
max             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
kurt            type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
cov             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
count           type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
corr            type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
apply           type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
aggregate       type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
min             type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>
median          type->   method                       ...dow=3,min_periods=3,center=True,axis=0]>

tmp1 = pd.DataFrame(np.random.rand(10, 100))

df_rolling = tmp1.rolling(window=3, min_periods=3, )
utils.get_class_info(df_rolling)
